@@ -4,7 +4,7 @@ apt_changelog.py
 
 Replacement for apt changelog, can be used both standalone and as a module.
 
-In addition to apt changelog sources, it directly supports also Pearl Linux
+In addition to apt changelog sources, it directly supports also Linux Pearl
 and Launchpad repositories and will try to retrieve a changelog for packages
 from all other debian package sources as well. Most importantly, changelogs
 for installed packages are retrieved locally to avoid unnecessary network
@@ -139,8 +139,8 @@ Proceed with the download?"
         # parse the package's origin
         if not self.candidate.downloadable:
             origin = "local_package"
-        elif self.candidate.origin == "pearllinux":
-            origin = "pearllinux"
+        elif self.candidate.origin == "Pearl":
+            origin = "Pearl"
         elif self.candidate.origin.startswith("LP-PPA-"):
             origin = "LP-PPA"
         elif self.apt_origins and self.candidate.origin in self.apt_origins.list():
@@ -175,7 +175,7 @@ Proceed with the download?"
             if not r:
                 self.exit_on_fail(2)
         # Pearl Linux repo
-        elif origin == 'pearllinux':
+        elif origin == 'Pearl':
             # Pearl repos don't have .debian.tar.xz files, only full packages, so
             # check the package cache first
             base_uri, _ = os.path.split(self.candidate.uri)
@@ -184,7 +184,7 @@ Proceed with the download?"
                 # fall back to last change info for the source package
                 # Pearl's naming scheme seems to be using amd64 unless source
                 # is i386 only, we always check amd64 first
-                base_uri = "http://packages.pearllinux.net/dev/%s_%s_%s.changes"
+                base_uri = "http://packages.pearllinux.com/dev/%s_%s_%s.changes"
                 uri = base_uri % (self.candidate.source_name,
                     self.candidate.source_version, "amd64")
                 r = self.check_url(uri, False)
@@ -197,13 +197,13 @@ Proceed with the download?"
 
         # Launchpad PPA
         elif origin == 'LP-PPA':
-            ppa_url, ppa_owner, ppa_name, _ = \
-                self.candidate.uri.split("://")[1].split("/", 3)
-            base_uri = "https://ppa.launchpadcontent.net/%s/%s/ubuntu/pool/main/{self.source_prefix()}/%s" % (ppa_owner, ppa_name, self.candidate.source_name)
+            ppa_owner, ppa_name, _ = \
+                self.candidate.uri.split("ppa.launchpad.net/")[1].split("/", 2)
+            base_uri = f"http://ppa.launchpad.net/{ppa_owner}/{ppa_name}/ubuntu/pool/main/{self.source_prefix()}/{self.candidate.source_name}"
             r, uri = self.get_changelog_uri(base_uri)
             if not r:
                 # fall back to last change info only
-                uri = "https://launchpad.net/~%s/+archive/ubuntu/%s/+files/%s_%s_source.changes" % (ppa_owner, ppa_name, self.candidate.source_name, self.candidate.source_version)
+                uri = f"https://launchpad.net/~{ppa_owner}/+archive/ubuntu/{ppa_name}/+files/{self.candidate.source_name}_{self.candidate.source_version}_source.changes"
                 r = self.check_url(uri, False)
                 if not r:
                     self.exit_on_fail(4)
@@ -337,7 +337,7 @@ Proceed with the download?"
                         if "/debian/changelog" in lines[0]:
                             for line in lines[2:]:
                                 if line.startswith("+"):
-                                    changelog += "%s\n" % line[1:]
+                                    changelog += f"{line[1:]}\n"
                                 else:
                                     break
                     if not changelog:
@@ -386,7 +386,7 @@ Proceed with the download?"
         if pkg_name in self.apt_cache:
             pkg = self.apt_cache[pkg_name]
         else:
-            print("E: Unable to locate package %s" % pkg_name, file=sys.stderr)
+            print(f"E: Unable to locate package {pkg_name}", file=sys.stderr)
             self.close(13)
 
         # get package data
@@ -407,10 +407,10 @@ Proceed with the download?"
                     break
             if not match_found:
                 if pkg_release:
-                    print('E: Release "%s" is unavailable for "%s"' % (pkg_release, pkg.name),
+                    print(f'E: Release "{pkg_release}" is unavailable for "{pkg.name}"',
                           file=sys.stderr)
                 else:
-                    print('E: Version "%s" is unavailable for "%s"' % (pkg_version, pkg.name),
+                    print(f'E: Version {pkg_version}" is unavailable for "{pkg.name}"',
                           file=sys.stderr)
                 self.close(14)
         else:
@@ -473,10 +473,10 @@ Proceed with the download?"
     def exit_on_fail(self, err:int=404):
         """ Prints error message and calls self.close() """
         try:
-            details = "Changelog unavailable for %s=%s" % (self.candidate.source_name, self.candidate.source_version_raw)
+            details = f"Changelog unavailable for {self.candidate.source_name}={self.candidate.source_version_raw}"
         except AttributeError:
             details = ""
-        print("E: Failed to fetch changelog. %s" % details, file=sys.stderr)
+        print(f"E: Failed to fetch changelog. {details}", file=sys.stderr)
         self.close(err)
 
     @staticmethod
@@ -487,14 +487,14 @@ Proceed with the download?"
         elif val in ('n', 'no'):
             return False
         else:
-            raise ValueError("Invalid response value %s" % val)
+            raise ValueError(f"Invalid response value {val}")
 
     def user_confirm(self, q:str):
         """ returns bool (always False in non-interactive mode) """
         if not self.interactive:
             if _DEBUG: print("Maximum size exceeded, skipping in non-interactive mode")
             return False
-        print("%s [y/n] " % q, end="")
+        print(f"{q} [y/n] ", end="")
         while True:
             try:
                 response = self.strtobool(input())
@@ -546,7 +546,7 @@ Proceed with the download?"
         """
         files = [s for s in filelist if "changelog" in s.lower()]
         if local:
-            testpath = "/usr/share/doc/%s/changelog" % self.candidate.name
+            testpath = f"/usr/share/doc/{self.candidate.name}/changelog"
             for item in files:
                 if item.lower().startswith(testpath):
                     return item
@@ -557,7 +557,7 @@ Proceed with the download?"
             testpath = "recipe/debian/changelog"
             if testpath in files:
                 return testpath
-            testpath = "usr/share/doc/%s/changelog" % self.candidate.name
+            testpath = f"usr/share/doc/{self.candidate.name}/changelog"
             for item in files:
                 if item.lower().startswith(testpath):
                     return item
@@ -665,13 +665,13 @@ Proceed with the download?"
         # XXX:  For APT sources we could just read the apt_pkg.SourceRecords()
         #       directly, if available, which it is not for most users, so
         #       probably not worth it
-        target_filename = self.parse_dsc("%s/%s_%s.dsc" % (base_uri, self.candidate.source_name, self.candidate.source_version))
+        target_filename = self.parse_dsc(f"{base_uri}/{self.candidate.source_name}_{self.candidate.source_version}.dsc")
         # get .debian.tar.xz or .diff.gz as a priority as the smallest options
         if (base_uri and target_filename and (
                 target_filename.lower().endswith('.debian.tar.xz') or
                 target_filename.lower().endswith('.diff.gz')
             )):
-            uri = "%s/%s" % (base_uri, target_filename)
+            uri = f"{base_uri}/{target_filename}"
             target_filename = None
             r = self.check_url(uri, msg = self.max_download_size_msg)
         else:
@@ -683,7 +683,7 @@ Proceed with the download?"
                 # cache miss, download the full source package or the .deb,
                 # depending on size and availability
                 if target_filename:
-                    uri_tar = "%s/%s" % (base_uri, target_filename)
+                    uri_tar = f"{base_uri}/{target_filename}"
                 else:
                     uri_tar = None
                 r, uri = self.get_deb_or_tar(uri_tar)
@@ -692,7 +692,8 @@ Proceed with the download?"
 def _generic_exception_handler(e):
     if _DEBUG:
         import traceback
-        print("%s: %s\n" % (e.__class__.__name__, traceback.format_exc()), file=sys.stderr)
+        print(f"{e.__class__.__name__}: {traceback.format_exc()}\n",
+              file=sys.stderr)
 
 def drop_cache():
     """ Drop the apt cache to free up memory. """
