@@ -1,34 +1,30 @@
 #!/usr/bin/python3
 import threading
+import signal
 import time
-import tempfile
 
 import gi
-gi.require_version('AppStreamGlib', '1.0')
-gi.require_version('Gtk', '3.0')
-from gi.repository import GLib, GObject, Gio, Gtk
+gi.require_version('AppStream', '1.0')
+from gi.repository import GLib, GObject, Gio
 
-from . import cache, _flatpak, _apt, dialogs
-from .misc import print_timing, check_ml, debug, warn
+from . import cache, _flatpak, _apt
+from .misc import print_timing
 
-PKG_TYPE_ALL = None
+PKG_TYPE_NONE = None
 PKG_TYPE_APT = "a"
 PKG_TYPE_FLATPAK = "f"
 
-Gtk.IconTheme.get_default().append_search_path("/usr/share/pearllinux/icons")
+# the action that initiated the task
+INSTALL_TASK = "install"
+UNINSTALL_TASK = "remove"
+UPDATE_TASK = "update"
 
 class InstallerTask:
-    # task types
-    INSTALL_TASK = "install"
-    UNINSTALL_TASK = "remove"
-    UPDATE_TASK = "update"
-
     # Set after a package selection, reflects whether task can proceed or not
     STATUS_NONE = "none"
     STATUS_OK = "ok"
     STATUS_BROKEN = "broken"
     STATUS_FORBIDDEN = "forbidden"
-    STATUS_UNKNOWN = "unknown"
 
     # Used by standalone progress window to update labels appropriately
     PROGRESS_STATE_INIT = "init"
@@ -38,46 +34,26 @@ class InstallerTask:
     PROGRESS_STATE_FINISHED = "finished"
     PROGRESS_STATE_FAILED = "failed"
 
-    def __init__(self, pkginfo, installer,
-                client_info_ready_callback, client_info_error_callback,
-                client_installer_finished_cb, client_installer_progress_cb,
-                installer_cleanup_cb, installer_error_cleanup_cb, is_addon_task=False, use_mainloop=False, parent_window=None):
-        self.type = InstallerTask.INSTALL_TASK
+    def __init__(self, pkginfo, installer, info_ready_callback):
+        self.type = INSTALL_TASK
 
-        self.use_mainloop = use_mainloop
-        self.parent_window = parent_window
+        self.parent_window = None
 
         # pkginfo will be None for an update task
         self.pkginfo = pkginfo
-        self.is_addon_task = is_addon_task
-
-        # AsApp if available
-        self.asapp = None
 
         self.name = None
 
-        # Lists the refs we selected for an update, vs those added as dependencies (this is for our confirm dialog to
-        # make layout decisions)
-        self.initial_refs_to_update = []
-
-        # FlatpakRef : pre-transaction-version or commit
-        # For activity logging so we can record pre and post-version.
-        self.ref_prior_versions_dict = {}
-        self.transaction_log = []
-
         # Set by .select_pkginfo(), the re-entry point after a task is fully
         # calculated, and the UI should be updated with detailed info about
         # the pending operation (disk use, etc..)
-        self.info_ready_callback = client_info_ready_callback
-        self.info_error_callback = client_info_error_callback
+        self.info_ready_callback = info_ready_callback
         # To be checked by the info_ready_callback, to allow the UI to reflect
         # the ability to proceed with a task, or report that something is not right.
         self.info_ready_status = self.STATUS_NONE
 
-        # Set by the backend, the functions to call to actually confirm and perform the
-        # task (will be none on STATUS_BROKEN or _FORBIDDEN)
-        self.confirm = lambda: True
-        self.cancel = lambda: True
+        # Set by the backend, the function to call to actually perform the task (will
+        # be none on STATUS_BROKEN or _FORBIDDEN)
         self.execute = None
 
         # Passed to _flatpak operations to respond to the Cancel button in the
@@ -88,14 +64,14 @@ class InstallerTask:
         # The .client_* callbacks are arguments of Installer.execute_task().  The
         # client_finished_cb is required.  If the progress callback is missing, a standalone
         # progress window will be provided.
-        self.client_progress_cb = client_installer_progress_cb
-        self.client_finished_cb = client_installer_finished_cb
+        self.client_progress_cb = None
+        self.client_finished_cb = None
 
         # These are internally used - called as the 'real' error and finished callback,
         # to do some cleanup like removing the task and reloading the apt cache before
         # finally calling task.client_finished_cb
-        self.error_cleanup_cb = installer_error_cleanup_cb
-        self.finished_cleanup_cb = installer_cleanup_cb
+        self.error_cleanup_cb = None
+        self.finished_cleanup_cb = None
 
         self.has_window = False
         # Updated throughout a flatpak operation - for now it's used for updating the
@@ -106,22 +82,18 @@ class InstallerTask:
         # The error message displayed in a popup if a flatpak operation fails.
         self.error_message = None
 
+        # apt only, stores the AptTransaction
         self.transaction = None
-        self.pkit_request_id = 0
 
         # The command that can be used to launch the current target package, if it's installed
         self.exec_string = None
 
         # List of additional packages to install, remove or update, based on the selected
-        # pkginfo. Depending on the backend, they will consist of PkPackages or stringified
-        # flatpak refs (the result of ref.format_ref()).
+        # pkginfo. Always lists of strings, but depending on the backend, they will consist
+        # of package names (apt) or stringified flatpak refs (the result of ref.format_ref())
         self.to_install = []
-        self.to_reinstall = [] # unused
         self.to_remove = []
-        self.to_purge = [] # unused
         self.to_update = []
-        self.to_downgrade = [] # unused
-        self.to_skip_upgrade = [] # unused
 
         # Size info for display, calculated by the backend during .select_pkginfo()
         self.download_size = 0
@@ -141,116 +113,53 @@ class InstallerTask:
                 self.remote = pkginfo.remote
                 self.branch = pkginfo.branch
 
-    def set_version(self, installer):
-        if self.type == InstallerTask.INSTALL_TASK:
-            # install packages, show pending version
-            self.version = installer.get_version(self.pkginfo)
-        else:
-            # Remove packages, show current version
-            self.version = installer.get_installed_version(self.pkginfo)
-
-    def get_transaction_log(self):
-        return self.transaction_log
-
-    def call_info_ready_callback(self):
-        if self.info_ready_callback == None:
-            return
-
-        if self.use_mainloop:
-            GLib.idle_add(self.info_ready_callback, self, priority=GLib.PRIORITY_DEFAULT)
-        else:
-            self.info_ready_callback(self)
-
-    def handle_error(self, error, info_stage=False):
-        try:
-            self.error_message = error.message
-        except:
-            self.error_message = str(error)
-
-        if info_stage:
-            if self.info_error_callback == None:
-                dialogs.show_error(self.error_message, self.parent_window)
-                return
-
-            if self.use_mainloop:
-                GLib.idle_add(self.info_error_callback, self, priority=GLib.PRIORITY_DEFAULT)
-            else:
-                self.info_error_callback(self)
-        else:
-            dialogs.show_error(self.error_message, self.parent_window)
-
-    def call_finished_cleanup_callback(self):
-        if not self.finished_cleanup_cb:
-            return
-
-        if self.use_mainloop:
-            GLib.idle_add(self.finished_cleanup_cb, self, priority=GLib.PRIORITY_DEFAULT)
-        else:
-            self.finished_cleanup_cb(self)
-
-    def call_error_cleanup_callback(self):
-        if not self.error_cleanup_cb:
-            return
-
-        if self.use_mainloop:
-            GLib.idle_add(self.error_cleanup_cb, self, priority=GLib.PRIORITY_DEFAULT)
-        else:
-            self.error_cleanup_cb(self)
+            self.version = installer.get_version(pkginfo)
 
 class Installer:
-    def __init__(self, pkg_type=PKG_TYPE_ALL, temp=False):
+    def __init__(self):
         self.tasks = {}
-        self.pkg_type = pkg_type
-
-        if temp:
-            f = tempfile.NamedTemporaryFile(prefix="pearl-common-installer-tmp")
-            self.cache_path = f.name
-        else:
-            self.cache_path = None
 
         self.remotes_changed = False
         self.inited = False
 
         self.have_flatpak = False
-        self.have_flatpak = self._get_flatpak_status()
+        self._determine_flatpak_status()
 
         self.cache = {}
         self._init_cb = None
 
         self.startup_timer = time.time()
 
-    def _get_flatpak_status(self):
+    def _determine_flatpak_status(self):
         try:
             gi.require_version('Flatpak', '1.0')
             from gi.repository import Flatpak
 
-            return True
-        except:
-            warn("No flatpak support, install flatpak and gir1.2-flatpak-1.0 and restart pearlinstall to enable it.")
+            self.have_flatpak = True
 
-        return False
+            return
+        except:
+            print("No flatpak support, install flatpak and gir1.2-flatpak-1.0 and restart pearlinstall to enable it.")
+
+        self.have_flatpak = False
 
     def init_sync(self):
         """
-        Loads the cache synchronously.  Returns True if all went ok, and returns False if there
+        Loads the cache asynchronously.  Returns True if all went ok, and returns False if there
         is no cache (or it's too old.)  You should then call init() with a callback so the cache
         can be regenerated.
         """
-        
-        if self.pkg_type == PKG_TYPE_FLATPAK and not self.have_flatpak:
-            debug("Not syncing for flatpaks only, as there is currently no support")
-            return True
-
         self.settings = Gio.Settings(schema_id="com.pearllinux.install")
 
         if self._fp_remotes_have_changed():
             self.remotes_changed = True
+            return False
 
         self.backend_table = {}
 
-        self.cache = cache.PkgCache(self.pkg_type, self.cache_path, self.have_flatpak)
+        self.cache = cache.PkgCache(self.have_flatpak)
 
-        if self.cache.status == self.cache.STATUS_OK and not self.remotes_changed:
+        if self.cache.status == self.cache.STATUS_OK:
             self.inited = True
 
             GObject.idle_add(self.initialize_appstream)
@@ -267,7 +176,7 @@ class Installer:
         """
         self.backend_table = {}
 
-        self.cache = cache.PkgCache(self.pkg_type, self.cache_path, self.have_flatpak)
+        self.cache = cache.PkgCache(self.have_flatpak)
 
         self._init_cb = ready_callback
 
@@ -277,7 +186,7 @@ class Installer:
             GObject.idle_add(self._idle_cache_load_done)
         else:
             if self.remotes_changed:
-                debug("Installer: Flatpak remotes have changed, forcing a new cache.")
+                print("PearlInstall: Flatpak remotes have changed, forcing a new cache.")
 
             self.cache.force_new_cache_async(self._idle_cache_load_done)
 
@@ -289,12 +198,6 @@ class Installer:
         """
         self.cache.force_new_cache_async(ready_callback)
 
-    def force_new_cache_sync(self):
-        """
-        Forces the cache to regenerate synchronously
-        """
-        self.cache.force_new_cache()
-
     def _idle_cache_load_done(self):
         self.inited = True
 
@@ -306,7 +209,7 @@ class Installer:
 
         self.generate_uncached_pkginfos(self.cache)
 
-        debug('Full installer startup took %0.3f ms' % ((time.time() - self.startup_timer) * 1000.0))
+        print('Full installer startup took %0.3f ms' % ((time.time() - self.startup_timer) * 1000.0))
 
         if self._init_cb:
             self._init_cb()
@@ -320,12 +223,15 @@ class Installer:
         installed apps, plus if you uninstall the app, the remote gets auto-removed.
         """
         changed = False
-        real_remote_count = 0
+        real_remote_count = 0;
 
         saved_remotes = self.settings.get_strv("flatpak-remotes")
         fp_remotes = self.list_flatpak_remotes()
 
         for remote_info in fp_remotes:
+            if remote_info.noenumerate:
+                continue
+
             real_remote_count += 1
 
             item = "%s::%s::%s" % (remote_info.name, remote_info.url, str(remote_info.disabled))
@@ -347,16 +253,16 @@ class Installer:
         fp_remotes = self.list_flatpak_remotes()
 
         for remote_info in fp_remotes:
+            if remote_info.noenumerate:
+                continue
+
             item = "%s::%s::%s" % (remote_info.name, remote_info.url, str(remote_info.disabled))
 
             new_remotes.append(item)
 
         self.settings.set_strv("flatpak-remotes", new_remotes)
 
-    def select_pkginfo(self, pkginfo,
-                       client_info_ready_callback, client_info_error_callback,
-                       client_installer_finished_cb, client_installer_progress_cb,
-                       use_mainloop=False, parent_window=None):
+    def select_pkginfo(self, pkginfo, ready_callback):
         """
         Initiates calculations for installing or removing a particular package
         (depending upon whether or not the selected package is installed.  Creates
@@ -372,80 +278,34 @@ class Installer:
             task = self.tasks[pkginfo.pkg_hash]
 
             GObject.idle_add(task.info_ready_callback, task)
-            return task.cancellable
+            return
 
-        task = InstallerTask(pkginfo, self,
-                             client_info_ready_callback, client_info_error_callback,
-                             client_installer_finished_cb, client_installer_progress_cb,
-                             self._task_finished, self._task_error,
-                             use_mainloop=use_mainloop, parent_window=parent_window)
+        task = InstallerTask(pkginfo, self, ready_callback)
 
         if self.pkginfo_is_installed(pkginfo):
             # It's not installed, so assume we're installing
-            task.type = InstallerTask.UNINSTALL_TASK
+            task.type = UNINSTALL_TASK
         else:
-            task.type = InstallerTask.INSTALL_TASK
-
-        task.set_version(self)
-        task.asapp = self.get_appstream_app_for_pkginfo(pkginfo)
+            task.type = INSTALL_TASK
 
         if pkginfo.pkg_hash.startswith("a"):
             _apt.select_packages(task)
         else:
             _flatpak.select_packages(task)
 
-        return task.cancellable
-
-    def select_flatpak_updates(self, refs,
-                               client_info_ready_callback, client_info_error_callback,
-                               client_installer_finished_cb, client_installer_progress_cb,
-                               use_mainloop=False):
+    def prepare_flatpak_update(self, ready_callback):
         """
         Creates an InstallerTask populated with all flatpak packages that can be
-        updated. If refs is empty, selects all possible updates, otherwise, only attempts
-        to update refs.
+        updated.  Note, unlike select_pkginfo, there is no 'primary' package here.
+        Only disk utilization and download info, along with the list of ref strings
+        (in task.to_update) will be populated.
         """
-        task = InstallerTask(None, self,
-                             client_info_ready_callback, client_info_error_callback,
-                             client_installer_finished_cb, client_installer_progress_cb,
-                             self._task_finished, self._task_error,
-                             use_mainloop=use_mainloop)
+        task = InstallerTask(None, self, ready_callback)
 
-        task.type = InstallerTask.UPDATE_TASK
-        task.initial_refs_to_update = refs if refs else []
+        task.type = UPDATE_TASK
 
         _flatpak.select_updates(task)
 
-
-    def create_addon_task(self, comp, remote_name, remote_url,
-                          client_info_ready_callback, client_info_error_callback,
-                          client_installer_finished_cb, client_installer_progress_cb,
-                          use_mainloop=False):
-
-        def pkginfo_ready(pkginfo):
-            task = InstallerTask(pkginfo, self,
-                                 client_info_ready_callback, client_info_error_callback,
-                                 client_installer_finished_cb, client_installer_progress_cb,
-                                 self._task_finished, self._task_error,
-                                 is_addon_task=True,
-                                 use_mainloop=use_mainloop)
-
-            if pkginfo.installed:
-                task.type = InstallerTask.UNINSTALL_TASK
-            else:
-                task.type = InstallerTask.INSTALL_TASK
-
-            task.set_version(self)
-
-            _flatpak.select_packages(task)
-
-        def create_pkginfo_thread(comp, remote_name, remote_url):
-            pkginfo = _flatpak.create_pkginfo_from_as_component(comp, remote_name, remote_url)
-            GLib.idle_add(pkginfo_ready, pkginfo, priority=GLib.PRIORITY_DEFAULT)
-
-        t = threading.Thread(target=create_pkginfo_thread, args=(comp, remote_name, remote_url))
-        t.start()
-
     def list_updated_flatpak_pkginfos(self):
         """
         Returns a list of flatpak pkginfos that can be updated.  Unlike
@@ -454,12 +314,12 @@ class Installer:
         """
         return _flatpak.list_updated_pkginfos(self.cache)
 
-    def find_pkginfo(self, name, pkg_type=PKG_TYPE_ALL, remote=None):
+    def find_pkginfo(self, name, pkg_type=PKG_TYPE_NONE):
         """
         Attempts to find and return a PkgInfo object, given a package name.  If
         pkg_type is None, looks in first apt, then flatpaks.
         """
-        return self.cache.find_pkginfo(name, pkg_type, remote)
+        return self.cache.find_pkginfo(name, pkg_type)
 
     def get_pkginfo_from_ref_file(self, file, ready_callback):
         """
@@ -493,14 +353,6 @@ class Installer:
         else:
             return []
 
-    def get_remote_info_for_name(self, remote_name):
-        if self.have_flatpak:
-            for remote in _flatpak.list_remotes():
-                if remote.name == remote_name:
-                    return remote
-
-        return []
-
     def pkginfo_is_installed(self, pkginfo):
         """
         Returns whether or not a given package is currently installed.  This uses
@@ -515,14 +367,14 @@ class Installer:
         return False
 
     @print_timing
-    def generate_uncached_pkginfos(self, unused=None):
+    def generate_uncached_pkginfos(self, cache):
         """
         Flatpaks installed from .flatpakref files may not actually be in the saved
         pkginfo cache, specifically, if they're added from no-enumerate-marked remotes.
         This gets run at startup to collect and generate their info.
         """
         if self.have_flatpak:
-            _flatpak.generate_uncached_pkginfos(self.cache)
+            _flatpak.generate_uncached_pkginfos(cache)
 
     @print_timing
     def initialize_appstream(self):
@@ -534,7 +386,7 @@ class Installer:
             _flatpak.initialize_appstream()
         # Is there any reason to use apt's appstream?
 
-    def get_appstream_app_for_pkginfo(self, pkginfo):
+    def _get_backend_component(self, pkginfo):
         try:
             backend_component = self.backend_table[pkginfo]
 
@@ -546,8 +398,7 @@ class Installer:
                 if self.have_flatpak:
                     backend_component = _flatpak.search_for_pkginfo_as_component(pkginfo)
 
-            if backend_component is not None:
-                self.backend_table[pkginfo] = backend_component
+            self.backend_table[pkginfo] = backend_component
 
             # It's possible at some point we'll refresh appstream at runtime, if so we'll
             # want to clear cached data so it can be re-fetched anew.  For now there's
@@ -559,59 +410,11 @@ class Installer:
 
             return backend_component
 
-    def get_flatpak_launchables(self, pkginfo):
-        """
-        Return the launchables associated with the AsApp for this pkginfo.
-        """
-
-        if pkginfo.pkg_hash.startswith("a"):
-            debug("launch_flatpak: pkginfo is not a flatpak")
-
-        comp = self.get_appstream_app_for_pkginfo(pkginfo)
-
-        if comp is None:
-            return None
-
-        launchables = comp.get_launchables()
-
-        if len(launchables) == 0:
-            return None
-
-        return launchables
-
-    def get_flatpak_root_path(self):
-        """
-        Return the root path for the flatpak installation (generally /var/lib/flatpak for system
-        and ~/.local/share/flatpak for user.
-        """
-
-        return _flatpak.get_fp_sys().get_path().get_path()
-
-    def get_addons(self, pkginfo):
-        """
-        Returns an array of app ids of names of available addons
-        """
-        
-        if pkginfo.pkg_hash.startswith("a"):
-            return None
-
-        comp = self.get_appstream_app_for_pkginfo(pkginfo)
-
-        if comp is None:
-            return None
-
-        addons = comp.get_addons()
-
-        if len(addons) == 0:
-            return None
-
-        return addons
-
     def get_display_name(self, pkginfo):
         """
         Returns the name of the package formatted for displaying
         """
-        comp = self.get_appstream_app_for_pkginfo(pkginfo)
+        comp = self._get_backend_component(pkginfo)
 
         return pkginfo.get_display_name(comp)
 
@@ -626,7 +429,7 @@ class Installer:
             except Exception:
                 pass
 
-        comp = self.get_appstream_app_for_pkginfo(pkginfo)
+        comp = self._get_backend_component(pkginfo)
 
         return pkginfo.get_summary(comp)
 
@@ -641,23 +444,23 @@ class Installer:
             except Exception:
                 pass
 
-        comp = self.get_appstream_app_for_pkginfo(pkginfo)
+        comp = self._get_backend_component(pkginfo)
 
         return pkginfo.get_description(comp)
 
-    def get_icon(self, pkginfo, size):
+    def get_icon(self, pkginfo):
         """
         Returns the icon name (or path) to display for the package
         """
-        comp = self.get_appstream_app_for_pkginfo(pkginfo)
+        comp = self._get_backend_component(pkginfo)
 
-        return pkginfo.get_icon(pkginfo, comp, size)
+        return pkginfo.get_icon(comp)
 
     def get_screenshots(self, pkginfo):
         """
         Returns a list of screenshot urls
         """
-        comp = self.get_appstream_app_for_pkginfo(pkginfo)
+        comp = self._get_backend_component(pkginfo)
 
         return pkginfo.get_screenshots(comp)
 
@@ -665,41 +468,19 @@ class Installer:
         """
         Returns the current version string, if available
         """
-        comp = self.get_appstream_app_for_pkginfo(pkginfo)
+        comp = self._get_backend_component(pkginfo)
 
         return pkginfo.get_version(comp)
 
-    def get_installed_version(self, pkginfo):
-        """
-        Returns the currently deployed version of a flatpak.
-        """
-        if pkginfo.pkg_hash.startswith("a"):
-            # apt packages we don't really need to make a distinction.
-            return self.get_version(pkginfo)
-        else:
-            # flatpak packages, the appstream component shows the latest version provided in the xml,
-            # not the actual installed version.
-            return _flatpak._get_deployed_version(pkginfo)
-
-    def get_homepage_url(self, pkginfo):
+    def get_url(self, pkginfo):
         """
         Returns the home page url for a package.  If there is
         no url for the package, in the case of flatpak, the remote's url
         is displayed instead
         """
-        comp = self.get_appstream_app_for_pkginfo(pkginfo)
+        comp = self._get_backend_component(pkginfo)
 
-        return pkginfo.get_homepage_url(comp)
-
-    def get_help_url(self, pkginfo):
-        """
-        Returns the help url for a package.  If there is
-        no url for the package, returns an empty string. Apt always returns
-        an empty string.
-        """
-        comp = self.get_appstream_app_for_pkginfo(pkginfo)
-
-        return pkginfo.get_help_url(comp)
+        return pkginfo.get_url(comp)
 
     def is_busy(self):
         return len(self.tasks.keys()) > 0
@@ -721,71 +502,38 @@ class Installer:
         """
         return task.pkginfo.pkg_hash in self.tasks.keys()
 
-    def confirm_task(self, task):
-        return task.confirm()
-
-    def cancel_task(self, task):
-        task.cancel()
-
-    def execute_task(self, task):
+    def execute_task(self, task, client_finished_cb, client_progress_cb=None):
         """
         Executes a given task.  The client_finished_cb is required always, to notify
         when the task completes. The progress and error callbacks are optional.  If
         they're left out, a standalone progress window is created to allow the user to
         see the task's progress (and cancel it if desired.)
         """
+        self.tasks[task.pkginfo.pkg_hash] = task
+        print("Starting task for package %s, type '%s'" % (task.pkginfo.pkg_hash, task.type))
 
-        if task.pkginfo != None:
-            key = task.pkginfo.pkg_hash
-        else:
-            key = "updates"
+        task.client_finished_cb = client_finished_cb
+        task.client_progress_cb = client_progress_cb
 
-        self.tasks[key] = task
+        task.finished_cleanup_cb = self._task_finished
+        task.error_cleanup_cb = self._task_error
 
-        debug("Starting task for package %s, type '%s'" % (key, task.type))
-
-        task.execute()
+        task.execute(task)
 
     def _task_finished(self, task):
-        if not task.pkginfo:
-            try:
-                del self.tasks["updates"]
-                debug("Done with update task (success)")
-            except:
-                pass
-        else:
-            key = task.pkginfo.pkg_hash
-
-            if key:
-                try:
-                    del self.tasks[key]
-                    debug("Done with task (success)", key)
-                except:
-                    pass
+        print("Done with task (success)", task.pkginfo.pkg_hash)
+        del self.tasks[task.pkginfo.pkg_hash]
 
         self._post_task_update(task)
 
     def _task_error(self, task):
-        if not task.pkginfo:
-            try:
-                del self.tasks["updates"]
-                debug("Done with update task (failure)")
-            except:
-                pass
-        else:
-            key = task.pkginfo.pkg_hash
-
-            if key:
-                try:
-                    del self.tasks[key]
-                    debug("Done with task (failure)", key)
-                except:
-                    pass
+        print("Done with task (error)", task.pkginfo.pkg_hash)
+        del self.tasks[task.pkginfo.pkg_hash]
 
         self._post_task_update(task)
 
     def _post_task_update(self, task):
-        if task.pkginfo and task.pkginfo.pkg_hash.startswith("a"):
+        if task.pkginfo.pkg_hash.startswith("a"):
             thread = threading.Thread(target=self._apt_post_task_update_thread, args=(task,))
             thread.start()
         else:
@@ -799,5 +547,23 @@ class Installer:
         self._run_client_callback(task)
 
     def _run_client_callback(self, task):
-        if task.client_finished_cb:
-            GObject.idle_add(task.client_finished_cb, task, priority=GLib.PRIORITY_DEFAULT)
+        GObject.idle_add(task.client_finished_cb, task.pkginfo, task.error_message)
+
+def interact():
+    import readline
+    import code
+    variables = globals().copy()
+    variables.update(locals())
+    shell = code.InteractiveConsole(variables)
+    shell.interact()
+
+# Debugging - you can run installer.py on its own, and test things with the Installer (i)
+
+if __name__ == "__main__":
+    signal.signal(signal.SIGINT, signal.SIG_DFL)
+
+    i = Installer()
+    i.init(ready_callback=interact)
+
+    ml = GLib.MainLoop.new(None, True)
+    ml.run()
